// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_BROWSER_PROCESS_INTERNALS_PROCESS_INTERNALS_MOJOM_H_
#define CONTENT_BROWSER_PROCESS_INTERNALS_PROCESS_INTERNALS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/browser/process_internals/process_internals.mojom-shared.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace mojom {
class ProcessInternalsHandler;
using ProcessInternalsHandlerPtr = mojo::InterfacePtr<ProcessInternalsHandler>;
using ProcessInternalsHandlerPtrInfo = mojo::InterfacePtrInfo<ProcessInternalsHandler>;
using ThreadSafeProcessInternalsHandlerPtr =
    mojo::ThreadSafeInterfacePtr<ProcessInternalsHandler>;
using ProcessInternalsHandlerRequest = mojo::InterfaceRequest<ProcessInternalsHandler>;
using ProcessInternalsHandlerAssociatedPtr =
    mojo::AssociatedInterfacePtr<ProcessInternalsHandler>;
using ThreadSafeProcessInternalsHandlerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ProcessInternalsHandler>;
using ProcessInternalsHandlerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ProcessInternalsHandler>;
using ProcessInternalsHandlerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ProcessInternalsHandler>;


class ProcessInternalsHandlerProxy;

template <typename ImplRefTraits>
class ProcessInternalsHandlerStub;

class ProcessInternalsHandlerRequestValidator;
class ProcessInternalsHandlerResponseValidator;

class  ProcessInternalsHandler
    : public ProcessInternalsHandlerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProcessInternalsHandlerProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProcessInternalsHandlerStub<ImplRefTraits>;

  using RequestValidator_ = ProcessInternalsHandlerRequestValidator;
  using ResponseValidator_ = ProcessInternalsHandlerResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetIsolationModeMinVersion = 0,
    kGetIsolatedOriginsSizeMinVersion = 0,
  };
  virtual ~ProcessInternalsHandler() {}


  using GetIsolationModeCallback = base::OnceCallback<void(const std::string&)>;
  virtual void GetIsolationMode(GetIsolationModeCallback callback) = 0;


  using GetIsolatedOriginsSizeCallback = base::OnceCallback<void(uint32_t)>;
  virtual void GetIsolatedOriginsSize(GetIsolatedOriginsSizeCallback callback) = 0;
};
class  ProcessInternalsHandlerInterceptorForTesting : public ProcessInternalsHandler {
  virtual ProcessInternalsHandler* GetForwardingInterface() = 0;
  void GetIsolationMode(GetIsolationModeCallback callback) override;
  void GetIsolatedOriginsSize(GetIsolatedOriginsSizeCallback callback) override;
};
class  ProcessInternalsHandlerAsyncWaiter {
 public:
  explicit ProcessInternalsHandlerAsyncWaiter(ProcessInternalsHandler* proxy);
  ~ProcessInternalsHandlerAsyncWaiter();
  void GetIsolationMode(
      std::string* out_mode);
  void GetIsolatedOriginsSize(
      uint32_t* out_size);

 private:
  ProcessInternalsHandler* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(ProcessInternalsHandlerAsyncWaiter);
};

class  ProcessInternalsHandlerProxy
    : public ProcessInternalsHandler {
 public:
  explicit ProcessInternalsHandlerProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetIsolationMode(GetIsolationModeCallback callback) final;
  void GetIsolatedOriginsSize(GetIsolatedOriginsSizeCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ProcessInternalsHandlerStubDispatch {
 public:
  static bool Accept(ProcessInternalsHandler* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProcessInternalsHandler* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProcessInternalsHandler>>
class ProcessInternalsHandlerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProcessInternalsHandlerStub() {}
  ~ProcessInternalsHandlerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessInternalsHandlerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessInternalsHandlerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ProcessInternalsHandlerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ProcessInternalsHandlerResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_BROWSER_PROCESS_INTERNALS_PROCESS_INTERNALS_MOJOM_H_