// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "device/vr/public/mojom/isolated_xr_service.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "device/vr/public/mojom/isolated_xr_service.mojom-shared-message-ids.h"
namespace device {
namespace mojom {
XRDeviceRuntimeSessionOptions::XRDeviceRuntimeSessionOptions()
    : immersive(),
      provide_passthrough_camera(),
      render_process_id(),
      render_frame_id(),
      has_user_activation(),
      use_legacy_webvr_render_path() {}

XRDeviceRuntimeSessionOptions::XRDeviceRuntimeSessionOptions(
    bool immersive_in,
    bool provide_passthrough_camera_in,
    int32_t render_process_id_in,
    int32_t render_frame_id_in,
    bool has_user_activation_in,
    bool use_legacy_webvr_render_path_in)
    : immersive(std::move(immersive_in)),
      provide_passthrough_camera(std::move(provide_passthrough_camera_in)),
      render_process_id(std::move(render_process_id_in)),
      render_frame_id(std::move(render_frame_id_in)),
      has_user_activation(std::move(has_user_activation_in)),
      use_legacy_webvr_render_path(std::move(use_legacy_webvr_render_path_in)) {}

XRDeviceRuntimeSessionOptions::~XRDeviceRuntimeSessionOptions() = default;
size_t XRDeviceRuntimeSessionOptions::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->immersive);
  seed = mojo::internal::Hash(seed, this->provide_passthrough_camera);
  seed = mojo::internal::Hash(seed, this->render_process_id);
  seed = mojo::internal::Hash(seed, this->render_frame_id);
  seed = mojo::internal::Hash(seed, this->has_user_activation);
  seed = mojo::internal::Hash(seed, this->use_legacy_webvr_render_path);
  return seed;
}

bool XRDeviceRuntimeSessionOptions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char XRSessionController::Name_[] = "device.mojom.XRSessionController";

XRSessionControllerProxy::XRSessionControllerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void XRSessionControllerProxy::SetFrameDataRestricted(
    bool in_restricted) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRSessionController::SetFrameDataRestricted");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRSessionController_SetFrameDataRestricted_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRSessionController_SetFrameDataRestricted_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->restricted = in_restricted;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRSessionController::Name_);
  message.set_method_name("SetFrameDataRestricted");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool XRSessionControllerStubDispatch::Accept(
    XRSessionController* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kXRSessionController_SetFrameDataRestricted_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRSessionController::SetFrameDataRestricted",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::XRSessionController_SetFrameDataRestricted_Params_Data* params =
          reinterpret_cast<internal::XRSessionController_SetFrameDataRestricted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_restricted{};
      XRSessionController_SetFrameDataRestricted_ParamsDataView input_data_view(params, &serialization_context);
      
      p_restricted = input_data_view.restricted();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRSessionController::SetFrameDataRestricted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetFrameDataRestricted(
std::move(p_restricted));
      return true;
    }
  }
  return false;
}

// static
bool XRSessionControllerStubDispatch::AcceptWithResponder(
    XRSessionController* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kXRSessionController_SetFrameDataRestricted_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRSessionController::SetFrameDataRestricted",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool XRSessionControllerRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "XRSessionController RequestValidator");

  switch (message->header()->name) {
    case internal::kXRSessionController_SetFrameDataRestricted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRSessionController_SetFrameDataRestricted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void XRSessionControllerInterceptorForTesting::SetFrameDataRestricted(bool restricted) {
  GetForwardingInterface()->SetFrameDataRestricted(std::move(restricted));
}
XRSessionControllerAsyncWaiter::XRSessionControllerAsyncWaiter(
    XRSessionController* proxy) : proxy_(proxy) {}

XRSessionControllerAsyncWaiter::~XRSessionControllerAsyncWaiter() = default;


const char XRRuntimeEventListener::Name_[] = "device.mojom.XRRuntimeEventListener";

class XRRuntimeEventListener_OnDeviceActivated_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  XRRuntimeEventListener_OnDeviceActivated_ForwardToCallback(
      XRRuntimeEventListener::OnDeviceActivatedCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  XRRuntimeEventListener::OnDeviceActivatedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(XRRuntimeEventListener_OnDeviceActivated_ForwardToCallback);
};

XRRuntimeEventListenerProxy::XRRuntimeEventListenerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void XRRuntimeEventListenerProxy::OnDisplayInfoChanged(
    ::device::mojom::VRDisplayInfoPtr in_display_info) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntimeEventListener::OnDisplayInfoChanged");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntimeEventListener_OnDisplayInfoChanged_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntimeEventListener_OnDisplayInfoChanged_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->display_info)::BaseType::BufferWriter
      display_info_writer;
  mojo::internal::Serialize<::device::mojom::VRDisplayInfoDataView>(
      in_display_info, buffer, &display_info_writer, &serialization_context);
  params->display_info.Set(
      display_info_writer.is_null() ? nullptr : display_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->display_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null display_info in XRRuntimeEventListener.OnDisplayInfoChanged request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntimeEventListener::Name_);
  message.set_method_name("OnDisplayInfoChanged");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void XRRuntimeEventListenerProxy::OnDeviceActivated(
    ::device::mojom::VRDisplayEventReason in_reason, OnDeviceActivatedCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntimeEventListener::OnDeviceActivated");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntimeEventListener_OnDeviceActivated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntimeEventListener_OnDeviceActivated_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::device::mojom::VRDisplayEventReason>(
      in_reason, &params->reason);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntimeEventListener::Name_);
  message.set_method_name("OnDeviceActivated");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new XRRuntimeEventListener_OnDeviceActivated_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void XRRuntimeEventListenerProxy::OnDeviceIdle(
    ::device::mojom::VRDisplayEventReason in_reason) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntimeEventListener::OnDeviceIdle");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntimeEventListener_OnDeviceIdle_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntimeEventListener_OnDeviceIdle_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::device::mojom::VRDisplayEventReason>(
      in_reason, &params->reason);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntimeEventListener::Name_);
  message.set_method_name("OnDeviceIdle");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void XRRuntimeEventListenerProxy::OnExitPresent(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntimeEventListener::OnExitPresent");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntimeEventListener_OnExitPresent_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntimeEventListener_OnExitPresent_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntimeEventListener::Name_);
  message.set_method_name("OnExitPresent");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder {
 public:
  static XRRuntimeEventListener::OnDeviceActivatedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder> proxy(
        new XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "XRRuntimeEventListener::OnDeviceActivatedCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      bool in_will_not_present);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder);
};

bool XRRuntimeEventListener_OnDeviceActivated_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "device::mojom::XRRuntimeEventListener::OnDeviceActivatedCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::XRRuntimeEventListener_OnDeviceActivated_ResponseParams_Data* params =
      reinterpret_cast<
          internal::XRRuntimeEventListener_OnDeviceActivated_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_will_not_present{};
  XRRuntimeEventListener_OnDeviceActivated_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_will_not_present = input_data_view.will_not_present();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "XRRuntimeEventListener::OnDeviceActivated response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_will_not_present));
  return true;
}

void XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder::Run(
    bool in_will_not_present) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntimeEventListener_OnDeviceActivated_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntimeEventListener_OnDeviceActivated_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->will_not_present = in_will_not_present;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDeviceActivatedCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntimeEventListener::Name_);
  message.set_method_name("OnDeviceActivated");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool XRRuntimeEventListenerStubDispatch::Accept(
    XRRuntimeEventListener* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kXRRuntimeEventListener_OnDisplayInfoChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDisplayInfoChanged",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::XRRuntimeEventListener_OnDisplayInfoChanged_Params_Data* params =
          reinterpret_cast<internal::XRRuntimeEventListener_OnDisplayInfoChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::device::mojom::VRDisplayInfoPtr p_display_info{};
      XRRuntimeEventListener_OnDisplayInfoChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDisplayInfo(&p_display_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntimeEventListener::OnDisplayInfoChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDisplayInfoChanged(
std::move(p_display_info));
      return true;
    }
    case internal::kXRRuntimeEventListener_OnDeviceActivated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDeviceActivated",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntimeEventListener_OnDeviceIdle_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDeviceIdle",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::XRRuntimeEventListener_OnDeviceIdle_Params_Data* params =
          reinterpret_cast<internal::XRRuntimeEventListener_OnDeviceIdle_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::device::mojom::VRDisplayEventReason p_reason{};
      XRRuntimeEventListener_OnDeviceIdle_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadReason(&p_reason))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntimeEventListener::OnDeviceIdle deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDeviceIdle(
std::move(p_reason));
      return true;
    }
    case internal::kXRRuntimeEventListener_OnExitPresent_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnExitPresent",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::XRRuntimeEventListener_OnExitPresent_Params_Data* params =
          reinterpret_cast<internal::XRRuntimeEventListener_OnExitPresent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      XRRuntimeEventListener_OnExitPresent_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntimeEventListener::OnExitPresent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnExitPresent();
      return true;
    }
  }
  return false;
}

// static
bool XRRuntimeEventListenerStubDispatch::AcceptWithResponder(
    XRRuntimeEventListener* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kXRRuntimeEventListener_OnDisplayInfoChanged_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDisplayInfoChanged",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntimeEventListener_OnDeviceActivated_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDeviceActivated",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::XRRuntimeEventListener_OnDeviceActivated_Params_Data* params =
          reinterpret_cast<
              internal::XRRuntimeEventListener_OnDeviceActivated_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::device::mojom::VRDisplayEventReason p_reason{};
      XRRuntimeEventListener_OnDeviceActivated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadReason(&p_reason))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntimeEventListener::OnDeviceActivated deserializer");
        return false;
      }
      XRRuntimeEventListener::OnDeviceActivatedCallback callback =
          XRRuntimeEventListener_OnDeviceActivated_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDeviceActivated(
std::move(p_reason), std::move(callback));
      return true;
    }
    case internal::kXRRuntimeEventListener_OnDeviceIdle_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnDeviceIdle",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntimeEventListener_OnExitPresent_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntimeEventListener::OnExitPresent",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool XRRuntimeEventListenerRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "XRRuntimeEventListener RequestValidator");

  switch (message->header()->name) {
    case internal::kXRRuntimeEventListener_OnDisplayInfoChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntimeEventListener_OnDisplayInfoChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntimeEventListener_OnDeviceActivated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntimeEventListener_OnDeviceActivated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntimeEventListener_OnDeviceIdle_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntimeEventListener_OnDeviceIdle_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntimeEventListener_OnExitPresent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntimeEventListener_OnExitPresent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool XRRuntimeEventListenerResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "XRRuntimeEventListener ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kXRRuntimeEventListener_OnDeviceActivated_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntimeEventListener_OnDeviceActivated_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void XRRuntimeEventListenerInterceptorForTesting::OnDisplayInfoChanged(::device::mojom::VRDisplayInfoPtr display_info) {
  GetForwardingInterface()->OnDisplayInfoChanged(std::move(display_info));
}
void XRRuntimeEventListenerInterceptorForTesting::OnDeviceActivated(::device::mojom::VRDisplayEventReason reason, OnDeviceActivatedCallback callback) {
  GetForwardingInterface()->OnDeviceActivated(std::move(reason), std::move(callback));
}
void XRRuntimeEventListenerInterceptorForTesting::OnDeviceIdle(::device::mojom::VRDisplayEventReason reason) {
  GetForwardingInterface()->OnDeviceIdle(std::move(reason));
}
void XRRuntimeEventListenerInterceptorForTesting::OnExitPresent() {
  GetForwardingInterface()->OnExitPresent();
}
XRRuntimeEventListenerAsyncWaiter::XRRuntimeEventListenerAsyncWaiter(
    XRRuntimeEventListener* proxy) : proxy_(proxy) {}

XRRuntimeEventListenerAsyncWaiter::~XRRuntimeEventListenerAsyncWaiter() = default;

void XRRuntimeEventListenerAsyncWaiter::OnDeviceActivated(
    ::device::mojom::VRDisplayEventReason reason, bool* out_will_not_present) {
  base::RunLoop loop;
  proxy_->OnDeviceActivated(std::move(reason),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_will_not_present
,
             bool will_not_present) {*out_will_not_present = std::move(will_not_present);
            loop->Quit();
          },
          &loop,
          out_will_not_present));
  loop.Run();
}

const char XRRuntime::Name_[] = "device.mojom.XRRuntime";

class XRRuntime_RequestSession_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  XRRuntime_RequestSession_ForwardToCallback(
      XRRuntime::RequestSessionCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  XRRuntime::RequestSessionCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(XRRuntime_RequestSession_ForwardToCallback);
};

class XRRuntime_RequestMagicWindowSession_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  XRRuntime_RequestMagicWindowSession_ForwardToCallback(
      XRRuntime::RequestMagicWindowSessionCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  XRRuntime::RequestMagicWindowSessionCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(XRRuntime_RequestMagicWindowSession_ForwardToCallback);
};

class XRRuntime_ListenToDeviceChanges_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  XRRuntime_ListenToDeviceChanges_ForwardToCallback(
      XRRuntime::ListenToDeviceChangesCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  XRRuntime::ListenToDeviceChangesCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(XRRuntime_ListenToDeviceChanges_ForwardToCallback);
};

XRRuntimeProxy::XRRuntimeProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void XRRuntimeProxy::RequestSession(
    XRDeviceRuntimeSessionOptionsPtr in_options, RequestSessionCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntime::RequestSession");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_RequestSession_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_RequestSession_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->options)::BaseType::BufferWriter
      options_writer;
  mojo::internal::Serialize<::device::mojom::XRDeviceRuntimeSessionOptionsDataView>(
      in_options, buffer, &options_writer, &serialization_context);
  params->options.Set(
      options_writer.is_null() ? nullptr : options_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->options.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null options in XRRuntime.RequestSession request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("RequestSession");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new XRRuntime_RequestSession_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void XRRuntimeProxy::RequestMagicWindowSession(
    RequestMagicWindowSessionCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntime::RequestMagicWindowSession");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_RequestMagicWindowSession_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_RequestMagicWindowSession_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("RequestMagicWindowSession");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new XRRuntime_RequestMagicWindowSession_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void XRRuntimeProxy::ListenToDeviceChanges(
    XRRuntimeEventListenerPtr in_listener, ListenToDeviceChangesCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntime::ListenToDeviceChanges");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_ListenToDeviceChanges_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_ListenToDeviceChanges_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::device::mojom::XRRuntimeEventListenerPtrDataView>(
      in_listener, &params->listener, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->listener),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid listener in XRRuntime.ListenToDeviceChanges request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("ListenToDeviceChanges");
#endif
  std::unique_ptr<mojo::MessageReceiver> responder(
      new XRRuntime_ListenToDeviceChanges_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void XRRuntimeProxy::SetListeningForActivate(
    bool in_listen_for_activation) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "device::mojom::XRRuntime::SetListeningForActivate");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_SetListeningForActivate_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_SetListeningForActivate_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->listen_for_activation = in_listen_for_activation;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("SetListeningForActivate");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class XRRuntime_RequestSession_ProxyToResponder {
 public:
  static XRRuntime::RequestSessionCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<XRRuntime_RequestSession_ProxyToResponder> proxy(
        new XRRuntime_RequestSession_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&XRRuntime_RequestSession_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~XRRuntime_RequestSession_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  XRRuntime_RequestSession_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "XRRuntime::RequestSessionCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      ::device::mojom::XRPresentationConnectionPtr in_connection, XRSessionControllerPtr in_controller);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(XRRuntime_RequestSession_ProxyToResponder);
};

bool XRRuntime_RequestSession_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "device::mojom::XRRuntime::RequestSessionCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::XRRuntime_RequestSession_ResponseParams_Data* params =
      reinterpret_cast<
          internal::XRRuntime_RequestSession_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::device::mojom::XRPresentationConnectionPtr p_connection{};
  XRSessionControllerPtr p_controller{};
  XRRuntime_RequestSession_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadConnection(&p_connection))
    success = false;
  p_controller =
      input_data_view.TakeController<decltype(p_controller)>();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "XRRuntime::RequestSession response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_connection), 
std::move(p_controller));
  return true;
}

void XRRuntime_RequestSession_ProxyToResponder::Run(
    ::device::mojom::XRPresentationConnectionPtr in_connection, XRSessionControllerPtr in_controller) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_RequestSession_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_RequestSession_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->connection)::BaseType::BufferWriter
      connection_writer;
  mojo::internal::Serialize<::device::mojom::XRPresentationConnectionDataView>(
      in_connection, buffer, &connection_writer, &serialization_context);
  params->connection.Set(
      connection_writer.is_null() ? nullptr : connection_writer.data());
  mojo::internal::Serialize<::device::mojom::XRSessionControllerPtrDataView>(
      in_controller, &params->controller, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestSessionCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("RequestSession");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class XRRuntime_RequestMagicWindowSession_ProxyToResponder {
 public:
  static XRRuntime::RequestMagicWindowSessionCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<XRRuntime_RequestMagicWindowSession_ProxyToResponder> proxy(
        new XRRuntime_RequestMagicWindowSession_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&XRRuntime_RequestMagicWindowSession_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~XRRuntime_RequestMagicWindowSession_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  XRRuntime_RequestMagicWindowSession_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "XRRuntime::RequestMagicWindowSessionCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      ::device::mojom::VRMagicWindowProviderPtr in_session, XRSessionControllerPtr in_controller);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(XRRuntime_RequestMagicWindowSession_ProxyToResponder);
};

bool XRRuntime_RequestMagicWindowSession_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "device::mojom::XRRuntime::RequestMagicWindowSessionCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::XRRuntime_RequestMagicWindowSession_ResponseParams_Data* params =
      reinterpret_cast<
          internal::XRRuntime_RequestMagicWindowSession_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::device::mojom::VRMagicWindowProviderPtr p_session{};
  XRSessionControllerPtr p_controller{};
  XRRuntime_RequestMagicWindowSession_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_session =
      input_data_view.TakeSession<decltype(p_session)>();
  p_controller =
      input_data_view.TakeController<decltype(p_controller)>();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "XRRuntime::RequestMagicWindowSession response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_session), 
std::move(p_controller));
  return true;
}

void XRRuntime_RequestMagicWindowSession_ProxyToResponder::Run(
    ::device::mojom::VRMagicWindowProviderPtr in_session, XRSessionControllerPtr in_controller) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_RequestMagicWindowSession_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_RequestMagicWindowSession_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::device::mojom::VRMagicWindowProviderPtrDataView>(
      in_session, &params->session, &serialization_context);
  mojo::internal::Serialize<::device::mojom::XRSessionControllerPtrDataView>(
      in_controller, &params->controller, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestMagicWindowSessionCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("RequestMagicWindowSession");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class XRRuntime_ListenToDeviceChanges_ProxyToResponder {
 public:
  static XRRuntime::ListenToDeviceChangesCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<XRRuntime_ListenToDeviceChanges_ProxyToResponder> proxy(
        new XRRuntime_ListenToDeviceChanges_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&XRRuntime_ListenToDeviceChanges_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~XRRuntime_ListenToDeviceChanges_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  XRRuntime_ListenToDeviceChanges_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "XRRuntime::ListenToDeviceChangesCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      ::device::mojom::VRDisplayInfoPtr in_display_info);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(XRRuntime_ListenToDeviceChanges_ProxyToResponder);
};

bool XRRuntime_ListenToDeviceChanges_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "device::mojom::XRRuntime::ListenToDeviceChangesCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::XRRuntime_ListenToDeviceChanges_ResponseParams_Data* params =
      reinterpret_cast<
          internal::XRRuntime_ListenToDeviceChanges_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::device::mojom::VRDisplayInfoPtr p_display_info{};
  XRRuntime_ListenToDeviceChanges_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadDisplayInfo(&p_display_info))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "XRRuntime::ListenToDeviceChanges response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_display_info));
  return true;
}

void XRRuntime_ListenToDeviceChanges_ProxyToResponder::Run(
    ::device::mojom::VRDisplayInfoPtr in_display_info) {
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kXRRuntime_ListenToDeviceChanges_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::device::mojom::internal::XRRuntime_ListenToDeviceChanges_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->display_info)::BaseType::BufferWriter
      display_info_writer;
  mojo::internal::Serialize<::device::mojom::VRDisplayInfoDataView>(
      in_display_info, buffer, &display_info_writer, &serialization_context);
  params->display_info.Set(
      display_info_writer.is_null() ? nullptr : display_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->display_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null display_info in ");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::ListenToDeviceChangesCallback",
               "message", message.name());
#endif

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(XRRuntime::Name_);
  message.set_method_name("ListenToDeviceChanges");
#endif

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool XRRuntimeStubDispatch::Accept(
    XRRuntime* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kXRRuntime_RequestSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestSession",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntime_RequestMagicWindowSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestMagicWindowSession",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntime_ListenToDeviceChanges_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::ListenToDeviceChanges",
               "message", message->name());
#endif
      break;
    }
    case internal::kXRRuntime_SetListeningForActivate_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::SetListeningForActivate",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::XRRuntime_SetListeningForActivate_Params_Data* params =
          reinterpret_cast<internal::XRRuntime_SetListeningForActivate_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_listen_for_activation{};
      XRRuntime_SetListeningForActivate_ParamsDataView input_data_view(params, &serialization_context);
      
      p_listen_for_activation = input_data_view.listen_for_activation();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntime::SetListeningForActivate deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetListeningForActivate(
std::move(p_listen_for_activation));
      return true;
    }
  }
  return false;
}

// static
bool XRRuntimeStubDispatch::AcceptWithResponder(
    XRRuntime* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kXRRuntime_RequestSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestSession",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::XRRuntime_RequestSession_Params_Data* params =
          reinterpret_cast<
              internal::XRRuntime_RequestSession_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      XRDeviceRuntimeSessionOptionsPtr p_options{};
      XRRuntime_RequestSession_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadOptions(&p_options))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntime::RequestSession deserializer");
        return false;
      }
      XRRuntime::RequestSessionCallback callback =
          XRRuntime_RequestSession_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestSession(
std::move(p_options), std::move(callback));
      return true;
    }
    case internal::kXRRuntime_RequestMagicWindowSession_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::RequestMagicWindowSession",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::XRRuntime_RequestMagicWindowSession_Params_Data* params =
          reinterpret_cast<
              internal::XRRuntime_RequestMagicWindowSession_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      XRRuntime_RequestMagicWindowSession_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntime::RequestMagicWindowSession deserializer");
        return false;
      }
      XRRuntime::RequestMagicWindowSessionCallback callback =
          XRRuntime_RequestMagicWindowSession_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestMagicWindowSession(std::move(callback));
      return true;
    }
    case internal::kXRRuntime_ListenToDeviceChanges_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::ListenToDeviceChanges",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      internal::XRRuntime_ListenToDeviceChanges_Params_Data* params =
          reinterpret_cast<
              internal::XRRuntime_ListenToDeviceChanges_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      XRRuntimeEventListenerPtr p_listener{};
      XRRuntime_ListenToDeviceChanges_ParamsDataView input_data_view(params, &serialization_context);
      
      p_listener =
          input_data_view.TakeListener<decltype(p_listener)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "XRRuntime::ListenToDeviceChanges deserializer");
        return false;
      }
      XRRuntime::ListenToDeviceChangesCallback callback =
          XRRuntime_ListenToDeviceChanges_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ListenToDeviceChanges(
std::move(p_listener), std::move(callback));
      return true;
    }
    case internal::kXRRuntime_SetListeningForActivate_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)device::mojom::XRRuntime::SetListeningForActivate",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool XRRuntimeRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "XRRuntime RequestValidator");

  switch (message->header()->name) {
    case internal::kXRRuntime_RequestSession_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_RequestSession_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntime_RequestMagicWindowSession_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_RequestMagicWindowSession_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntime_ListenToDeviceChanges_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_ListenToDeviceChanges_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntime_SetListeningForActivate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_SetListeningForActivate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool XRRuntimeResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "XRRuntime ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kXRRuntime_RequestSession_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_RequestSession_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntime_RequestMagicWindowSession_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_RequestMagicWindowSession_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kXRRuntime_ListenToDeviceChanges_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::XRRuntime_ListenToDeviceChanges_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void XRRuntimeInterceptorForTesting::RequestSession(XRDeviceRuntimeSessionOptionsPtr options, RequestSessionCallback callback) {
  GetForwardingInterface()->RequestSession(std::move(options), std::move(callback));
}
void XRRuntimeInterceptorForTesting::RequestMagicWindowSession(RequestMagicWindowSessionCallback callback) {
  GetForwardingInterface()->RequestMagicWindowSession(std::move(callback));
}
void XRRuntimeInterceptorForTesting::ListenToDeviceChanges(XRRuntimeEventListenerPtr listener, ListenToDeviceChangesCallback callback) {
  GetForwardingInterface()->ListenToDeviceChanges(std::move(listener), std::move(callback));
}
void XRRuntimeInterceptorForTesting::SetListeningForActivate(bool listen_for_activation) {
  GetForwardingInterface()->SetListeningForActivate(std::move(listen_for_activation));
}
XRRuntimeAsyncWaiter::XRRuntimeAsyncWaiter(
    XRRuntime* proxy) : proxy_(proxy) {}

XRRuntimeAsyncWaiter::~XRRuntimeAsyncWaiter() = default;

void XRRuntimeAsyncWaiter::RequestSession(
    XRDeviceRuntimeSessionOptionsPtr options, ::device::mojom::XRPresentationConnectionPtr* out_connection, XRSessionControllerPtr* out_controller) {
  base::RunLoop loop;
  proxy_->RequestSession(std::move(options),
      base::BindOnce(
          [](base::RunLoop* loop,
             ::device::mojom::XRPresentationConnectionPtr* out_connection
,
             XRSessionControllerPtr* out_controller
,
             ::device::mojom::XRPresentationConnectionPtr connection,
             XRSessionControllerPtr controller) {*out_connection = std::move(connection);*out_controller = std::move(controller);
            loop->Quit();
          },
          &loop,
          out_connection,
          out_controller));
  loop.Run();
}
void XRRuntimeAsyncWaiter::RequestMagicWindowSession(
    ::device::mojom::VRMagicWindowProviderPtr* out_session, XRSessionControllerPtr* out_controller) {
  base::RunLoop loop;
  proxy_->RequestMagicWindowSession(
      base::BindOnce(
          [](base::RunLoop* loop,
             ::device::mojom::VRMagicWindowProviderPtr* out_session
,
             XRSessionControllerPtr* out_controller
,
             ::device::mojom::VRMagicWindowProviderPtr session,
             XRSessionControllerPtr controller) {*out_session = std::move(session);*out_controller = std::move(controller);
            loop->Quit();
          },
          &loop,
          out_session,
          out_controller));
  loop.Run();
}
void XRRuntimeAsyncWaiter::ListenToDeviceChanges(
    XRRuntimeEventListenerPtr listener, ::device::mojom::VRDisplayInfoPtr* out_display_info) {
  base::RunLoop loop;
  proxy_->ListenToDeviceChanges(std::move(listener),
      base::BindOnce(
          [](base::RunLoop* loop,
             ::device::mojom::VRDisplayInfoPtr* out_display_info
,
             ::device::mojom::VRDisplayInfoPtr display_info) {*out_display_info = std::move(display_info);
            loop->Quit();
          },
          &loop,
          out_display_info));
  loop.Run();
}

}  // namespace mojom
}  // namespace device

namespace mojo {


// static
bool StructTraits<::device::mojom::XRDeviceRuntimeSessionOptions::DataView, ::device::mojom::XRDeviceRuntimeSessionOptionsPtr>::Read(
    ::device::mojom::XRDeviceRuntimeSessionOptions::DataView input,
    ::device::mojom::XRDeviceRuntimeSessionOptionsPtr* output) {
  bool success = true;
  ::device::mojom::XRDeviceRuntimeSessionOptionsPtr result(::device::mojom::XRDeviceRuntimeSessionOptions::New());
  
      result->immersive = input.immersive();
      result->provide_passthrough_camera = input.provide_passthrough_camera();
      result->render_process_id = input.render_process_id();
      result->render_frame_id = input.render_frame_id();
      result->has_user_activation = input.has_user_activation();
      result->use_legacy_webvr_render_path = input.use_legacy_webvr_render_path();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif