// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_FACTORY_MOJOM_H_
#define SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_FACTORY_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/content/public/mojom/navigable_contents_factory.mojom-shared.h"
#include "services/content/public/mojom/navigable_contents.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace content {
namespace mojom {
class NavigableContentsFactory;
using NavigableContentsFactoryPtr = mojo::InterfacePtr<NavigableContentsFactory>;
using NavigableContentsFactoryPtrInfo = mojo::InterfacePtrInfo<NavigableContentsFactory>;
using ThreadSafeNavigableContentsFactoryPtr =
    mojo::ThreadSafeInterfacePtr<NavigableContentsFactory>;
using NavigableContentsFactoryRequest = mojo::InterfaceRequest<NavigableContentsFactory>;
using NavigableContentsFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<NavigableContentsFactory>;
using ThreadSafeNavigableContentsFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NavigableContentsFactory>;
using NavigableContentsFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NavigableContentsFactory>;
using NavigableContentsFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NavigableContentsFactory>;

class NavigableContentsParams;
using NavigableContentsParamsPtr = mojo::InlinedStructPtr<NavigableContentsParams>;


class NavigableContentsFactoryProxy;

template <typename ImplRefTraits>
class NavigableContentsFactoryStub;

class NavigableContentsFactoryRequestValidator;

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactory
    : public NavigableContentsFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NavigableContentsFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = NavigableContentsFactoryStub<ImplRefTraits>;

  using RequestValidator_ = NavigableContentsFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateContentsMinVersion = 0,
  };
  virtual ~NavigableContentsFactory() {}

  virtual void CreateContents(NavigableContentsParamsPtr params, ::content::mojom::NavigableContentsRequest request, ::content::mojom::NavigableContentsClientPtr client) = 0;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactoryInterceptorForTesting : public NavigableContentsFactory {
  virtual NavigableContentsFactory* GetForwardingInterface() = 0;
  void CreateContents(NavigableContentsParamsPtr params, ::content::mojom::NavigableContentsRequest request, ::content::mojom::NavigableContentsClientPtr client) override;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactoryAsyncWaiter {
 public:
  explicit NavigableContentsFactoryAsyncWaiter(NavigableContentsFactory* proxy);
  ~NavigableContentsFactoryAsyncWaiter();

 private:
  NavigableContentsFactory* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NavigableContentsFactoryAsyncWaiter);
};

class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactoryProxy
    : public NavigableContentsFactory {
 public:
  explicit NavigableContentsFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateContents(NavigableContentsParamsPtr params, ::content::mojom::NavigableContentsRequest request, ::content::mojom::NavigableContentsClientPtr client) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactoryStubDispatch {
 public:
  static bool Accept(NavigableContentsFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NavigableContentsFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NavigableContentsFactory>>
class NavigableContentsFactoryStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NavigableContentsFactoryStub() {}
  ~NavigableContentsFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NavigableContentsFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsFactoryRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};


class COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) NavigableContentsParams {
 public:
  using DataView = NavigableContentsParamsDataView;
  using Data_ = internal::NavigableContentsParams_Data;

  template <typename... Args>
  static NavigableContentsParamsPtr New(Args&&... args) {
    return NavigableContentsParamsPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static NavigableContentsParamsPtr From(const U& u) {
    return mojo::TypeConverter<NavigableContentsParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, NavigableContentsParams>::Convert(*this);
  }


  NavigableContentsParams();

  ~NavigableContentsParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = NavigableContentsParamsPtr>
  NavigableContentsParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, NavigableContentsParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        NavigableContentsParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        NavigableContentsParams::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::NavigableContentsParams_UnserializedMessageContext<
            UserType, NavigableContentsParams::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<NavigableContentsParams::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return NavigableContentsParams::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::NavigableContentsParams_UnserializedMessageContext<
            UserType, NavigableContentsParams::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<NavigableContentsParams::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }


 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
NavigableContentsParamsPtr NavigableContentsParams::Clone() const {
  return New(
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, NavigableContentsParams>::value>::type*>
bool NavigableContentsParams::Equals(const T& other_struct) const {
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct COMPONENT_EXPORT(CONTENT_SERVICE_MOJOM) StructTraits<::content::mojom::NavigableContentsParams::DataView,
                                         ::content::mojom::NavigableContentsParamsPtr> {
  static bool IsNull(const ::content::mojom::NavigableContentsParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::NavigableContentsParamsPtr* output) { output->reset(); }

  static bool Read(::content::mojom::NavigableContentsParams::DataView input, ::content::mojom::NavigableContentsParamsPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_CONTENT_PUBLIC_MOJOM_NAVIGABLE_CONTENTS_FACTORY_MOJOM_H_