// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_DEVICE_PUBLIC_MOJOM_NFC_PROVIDER_MOJOM_H_
#define SERVICES_DEVICE_PUBLIC_MOJOM_NFC_PROVIDER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/device/public/mojom/nfc_provider.mojom-shared.h"
#include "services/device/public/mojom/nfc.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace device {
namespace mojom {
class NFCProvider;
using NFCProviderPtr = mojo::InterfacePtr<NFCProvider>;
using NFCProviderPtrInfo = mojo::InterfacePtrInfo<NFCProvider>;
using ThreadSafeNFCProviderPtr =
    mojo::ThreadSafeInterfacePtr<NFCProvider>;
using NFCProviderRequest = mojo::InterfaceRequest<NFCProvider>;
using NFCProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<NFCProvider>;
using ThreadSafeNFCProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NFCProvider>;
using NFCProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NFCProvider>;
using NFCProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NFCProvider>;


class NFCProviderProxy;

template <typename ImplRefTraits>
class NFCProviderStub;

class NFCProviderRequestValidator;

class  NFCProvider
    : public NFCProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NFCProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = NFCProviderStub<ImplRefTraits>;

  using RequestValidator_ = NFCProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetNFCForHostMinVersion = 0,
  };
  virtual ~NFCProvider() {}

  virtual void GetNFCForHost(int32_t host_id, ::device::mojom::NFCRequest nfc) = 0;
};
class  NFCProviderInterceptorForTesting : public NFCProvider {
  virtual NFCProvider* GetForwardingInterface() = 0;
  void GetNFCForHost(int32_t host_id, ::device::mojom::NFCRequest nfc) override;
};
class  NFCProviderAsyncWaiter {
 public:
  explicit NFCProviderAsyncWaiter(NFCProvider* proxy);
  ~NFCProviderAsyncWaiter();

 private:
  NFCProvider* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(NFCProviderAsyncWaiter);
};

class  NFCProviderProxy
    : public NFCProvider {
 public:
  explicit NFCProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetNFCForHost(int32_t host_id, ::device::mojom::NFCRequest nfc) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  NFCProviderStubDispatch {
 public:
  static bool Accept(NFCProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NFCProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NFCProvider>>
class NFCProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NFCProviderStub() {}
  ~NFCProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NFCProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NFCProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  NFCProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace device

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_DEVICE_PUBLIC_MOJOM_NFC_PROVIDER_MOJOM_H_