// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_REMOTING_EVENT_INJECTOR_MOJOM_SHARED_H_
#define SERVICES_UI_PUBLIC_INTERFACES_REMOTING_EVENT_INJECTOR_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/ui/public/interfaces/remoting_event_injector.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace ui {
namespace mojom {


}  // namespace mojom
}  // namespace ui

namespace mojo {
namespace internal {

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace mojom {

enum class InjectedMouseButtonType : int32_t {
  kLeft,
  kMiddle,
  kRight,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, InjectedMouseButtonType value) {
  switch(value) {
    case InjectedMouseButtonType::kLeft:
      return os << "InjectedMouseButtonType::kLeft";
    case InjectedMouseButtonType::kMiddle:
      return os << "InjectedMouseButtonType::kMiddle";
    case InjectedMouseButtonType::kRight:
      return os << "InjectedMouseButtonType::kRight";
    default:
      return os << "Unknown InjectedMouseButtonType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(InjectedMouseButtonType value) {
  return internal::InjectedMouseButtonType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class RemotingEventInjectorInterfaceBase {};

using RemotingEventInjectorPtrDataView =
    mojo::InterfacePtrDataView<RemotingEventInjectorInterfaceBase>;
using RemotingEventInjectorRequestDataView =
    mojo::InterfaceRequestDataView<RemotingEventInjectorInterfaceBase>;
using RemotingEventInjectorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<RemotingEventInjectorInterfaceBase>;
using RemotingEventInjectorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<RemotingEventInjectorInterfaceBase>;
class RemotingEventInjector_MoveCursorToLocationInPixels_ParamsDataView {
 public:
  RemotingEventInjector_MoveCursorToLocationInPixels_ParamsDataView() {}

  RemotingEventInjector_MoveCursorToLocationInPixels_ParamsDataView(
      internal::RemotingEventInjector_MoveCursorToLocationInPixels_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLocationDataView(
      ::gfx::mojom::PointFDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLocation(UserType* output) {
    auto* pointer = data_->location.Get();
    return mojo::internal::Deserialize<::gfx::mojom::PointFDataView>(
        pointer, output, context_);
  }
 private:
  internal::RemotingEventInjector_MoveCursorToLocationInPixels_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RemotingEventInjector_InjectMousePressOrRelease_ParamsDataView {
 public:
  RemotingEventInjector_InjectMousePressOrRelease_ParamsDataView() {}

  RemotingEventInjector_InjectMousePressOrRelease_ParamsDataView(
      internal::RemotingEventInjector_InjectMousePressOrRelease_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadButton(UserType* output) const {
    auto data_value = data_->button;
    return mojo::internal::Deserialize<::ui::mojom::InjectedMouseButtonType>(
        data_value, output);
  }

  InjectedMouseButtonType button() const {
    return static_cast<InjectedMouseButtonType>(data_->button);
  }
  bool down() const {
    return data_->down;
  }
 private:
  internal::RemotingEventInjector_InjectMousePressOrRelease_Params_Data* data_ = nullptr;
};

class RemotingEventInjector_InjectMouseWheelInPixels_ParamsDataView {
 public:
  RemotingEventInjector_InjectMouseWheelInPixels_ParamsDataView() {}

  RemotingEventInjector_InjectMouseWheelInPixels_ParamsDataView(
      internal::RemotingEventInjector_InjectMouseWheelInPixels_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t delta_x() const {
    return data_->delta_x;
  }
  int32_t delta_y() const {
    return data_->delta_y;
  }
 private:
  internal::RemotingEventInjector_InjectMouseWheelInPixels_Params_Data* data_ = nullptr;
};

class RemotingEventInjector_InjectKeyEvent_ParamsDataView {
 public:
  RemotingEventInjector_InjectKeyEvent_ParamsDataView() {}

  RemotingEventInjector_InjectKeyEvent_ParamsDataView(
      internal::RemotingEventInjector_InjectKeyEvent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  int32_t native_key_code() const {
    return data_->native_key_code;
  }
  bool down() const {
    return data_->down;
  }
  bool suppress_auto_repeat() const {
    return data_->suppress_auto_repeat;
  }
 private:
  internal::RemotingEventInjector_InjectKeyEvent_Params_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace ui

namespace std {

template <>
struct hash<::ui::mojom::InjectedMouseButtonType>
    : public mojo::internal::EnumHashImpl<::ui::mojom::InjectedMouseButtonType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::mojom::InjectedMouseButtonType, ::ui::mojom::InjectedMouseButtonType> {
  static ::ui::mojom::InjectedMouseButtonType ToMojom(::ui::mojom::InjectedMouseButtonType input) { return input; }
  static bool FromMojom(::ui::mojom::InjectedMouseButtonType input, ::ui::mojom::InjectedMouseButtonType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::InjectedMouseButtonType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::InjectedMouseButtonType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::InjectedMouseButtonType>(input), output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace mojom {

inline void RemotingEventInjector_MoveCursorToLocationInPixels_ParamsDataView::GetLocationDataView(
    ::gfx::mojom::PointFDataView* output) {
  auto pointer = data_->location.Get();
  *output = ::gfx::mojom::PointFDataView(pointer, context_);
}









}  // namespace mojom
}  // namespace ui

#endif  // SERVICES_UI_PUBLIC_INTERFACES_REMOTING_EVENT_INJECTOR_MOJOM_SHARED_H_
